import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleBand, scaleLinear, max } from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/NandithaSura/5fd0cec840199beb5a16d3f0a2feab1b/raw/fc32657ebe7a8d2083e61f41f7dcaf80e16032a0/cars.csv';
const width = 800;
const height = 400;
const margin = { top: 20, right: 30, bottom: 50, left: 60 };

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = (d) => {
      d.mpg = +d['mpg'];
      return d;
    };
    csv(csvUrl, row).then((data) => {
      setData(data);
    });
  }, []);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = scaleBand()
    .domain(data.map((d) => d.model))
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain([0, max(data, (d) => d.mpg)])
    .nice()
    .range([innerHeight, 0]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        {data.map((d) => (
          <rect
            key={d.model}
            x={xScale(d.model)}
            y={yScale(d.mpg)}
            width={xScale.bandwidth()}
            height={innerHeight - yScale(d.mpg)}
            fill="steelblue"
          />
        ))}
        <g className="axis" transform={`translate(0,0)`}>
          {yScale.ticks().map((tickValue) => (
            <g
              key={tickValue}
              transform={`translate(0,${yScale(tickValue)})`}
            >
              <line x2={innerWidth} stroke="#ccc" />
              <text
                x="-6"
                dy="0.32em"
                textAnchor="end"
                fill="#000"
                fontSize="12"
              >
                {tickValue}
              </text>
            </g>
          ))}
        </g>
        <g
          className="axis"
          transform={`translate(0,${innerHeight})`}
        >
          {xScale.domain().map((model) => (
            <g
              key={model}
              transform={`translate(${xScale(model) + xScale.bandwidth() / 2},${margin.bottom - 10})`}
            >
              <line y2="6" stroke="#000" />
              <text
                y="9"
                dy="0.71em"
                textAnchor="middle"
                fill="#000"
                transform="rotate(-45)"
                fontSize="10"
              >
                {model}
              </text>
            </g>
          ))}
        </g>
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
